@@ -1,20 +1,40 @@
+import html
+import os
 import uuid
 from datetime import datetime
 from enum import Enum
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 from bson.objectid import ObjectId
 from pydantic import UUID4, BaseModel, Field
 
-from msaDocModels import sdu, wdc
-from msaDocModels.sdu import (
-    SDUAttachment,
-    SDUContent,
-    SDUData,
-    SDUEmail,
-    SDULanguage,
-    SDUText,
-)
+from msaDocModels import wdc
+from msaDocModels.utils.htmlutils import sanitize
+
+
+def get_crlf() -> str:
+    """get's the OS Environment Variable for ``CR_LF``.
+    Default: ``\\n``
+    """
+    ret: str = os.getenv("CR_LF", "\n")
+    return ret
+
+
+def get_sentence_seperator() -> str:
+    """get's the OS Environment Variable for ``SENTENCE_SEPARATOR``.
+    Default: `` `` (Space/Blank)
+    """
+    ret: str = os.getenv("SENTENCE_SEPARATOR", " ")
+    return ret
+
+
+def get_cr_paragraph() -> str:
+    # CR_PARAGRAPH
+    """get's the OS Environment Variable for ``CR_PARAGRAPH``.
+    Default: ``\\n\\n``
+    """
+    ret: str = os.getenv("CR_PARAGRAPH", "\n\n")
+    return ret
 
 
 class TenantIdInput(BaseModel):
@@ -50,6 +70,530 @@ class DocumentInput(TextInput):
     document_id: Optional[UUID4]
 
 
+class SDUPageImage(BaseModel):
+    """Page Image Pydantic Model.
+
+    Storing the information about the Image representation of a Page.
+    """
+
+    id: int = -1  # ID = Page Index.
+    filepath_name: str = ""  # Filepath to the image on filesystem storage.
+    height: float = 0.0  # Image Height.
+    width: float = 0.0  # Image Width.
+    dpi: float = 0.0  # Picture DPI Resolution.
+    format: str = ""  # Image Format (png, jpg etc.).
+    mode: str = ""  # Image Mode.
+    layout: List = []  # Image Layout Information."""
+
+    class Config:
+        orm_mode = False
+
+
+class SDUEmail(BaseModel):
+    """Parsed EMail Pydantic Model."""
+
+    msg_id: str = ""
+    msg_from: str = ""
+    msg_to: str = ""
+    msg_cc: str = ""
+    msg_bcc: str = ""
+    msg_subject: str = ""
+    msg_sent_date: str = ""
+    msg_body: str = ""
+    seg_body: str = ""  # Segmented Body (Signature, etc.)
+    seg_sign: str = ""
+    msg_sender_ip: str = ""
+    msg_to_domains: str = ""
+    msg_received: List = []
+    msg_reply_to: str = ""
+    msg_timezone: str = ""
+    msg_headers: Dict = {}
+
+    class Config:
+        orm_mode = False
+
+
+class SDUDetailLanguage(BaseModel):
+    """Detailed Language Pydantic Model."""
+
+    multiple: bool = False
+    reliable: bool = False
+    bytes: int = -1
+    details: Optional[Tuple] = tuple()
+    vectors: Optional[Tuple] = tuple()
+
+    class Config:
+        orm_mode = False
+
+
+class SDULanguage(BaseModel):
+    """Detected Language Pydantic Model."""
+
+    code: str = "unknown"  # Short de, en etc.
+    lang: str = "unknown"  # Language name like german.
+    reliable: bool = False  # is the detected result reliable.
+    proportion: int = -1  # Proportion of the text in this language.
+    bytes: int = -1  # Bytes of the text in this language.
+    confidence: float = -1  # Confidence from 0.01 to 1.0.
+    winner: Optional[str] = None  # Selected overall Winner
+
+    class Config:
+        orm_mode = False
+
+
+class SDULanguageDetails(SDULanguage):
+    details: Optional[List] = list()  # Details of the top 3 detected languages.
+
+
+class SDUStatistic(BaseModel):
+    """Text Statistics Pydantic Model."""
+
+    avg_character_per_word: float = 0
+    avg_letter_per_word: float = 0
+    avg_sentence_length: float = 0
+    avg_syllables_per_word: float = 0
+    avg_sentence_per_word: float = 0
+    difficult_words: int = 0
+    lexicon_count: int = 0
+    long_word_count: int = 0
+    reading_time_s: float = 0
+    sentence_count: int = 0
+    paragraph_count: int = 0
+    reading_ease_score: float = 0
+    reading_ease: str = ""
+    grade: float = 0
+    smog: float = 0
+    coleman: float = 0
+    reading_index: float = 0
+    reading_score: float = 0
+    write_formula: float = 0
+    fog: float = 0
+    standard: str = ""
+    crawford: float = 0
+    gulpease_index: float = 0
+    osman: float = 0
+
+    class Config:
+        orm_mode = False
+
+
+class SDUSentence(BaseModel):
+    id: int = -1
+    text: str = ""
+    xpos: List[str] = []
+    upos: List[str] = []
+    tokens: List[str] = []
+    text_defaults: List = []
+    text_ner: List = []
+    text_nlp: List = []
+    text_ml: List = []
+    text_wsd: List = []
+
+    class Config:
+        orm_mode = False
+
+
+class SDUPDFElement(BaseModel):
+    line_id: int = -1
+    span_id: int = -1
+    flags: int = 0
+    bold: bool = False
+    italic: bool = False
+    font: str = ""
+    fontsize: float = 0.0
+    color: int = 0
+
+
+class SDUParagraph(BaseModel):
+    id: int = -1
+    sort: int = -1
+    nsen: int = 0
+    semantic_type: str = "text"
+    section: str = "body"
+    size_type: str = "body"
+    sentences: List[SDUSentence] = []
+    sentences_en: List[SDUSentence] = []
+    clean: str = ""
+    lang: SDULanguage = SDULanguage()
+    elements: List[SDUPDFElement] = []
+
+    class Config:
+        orm_mode = False
+
+    def has_text(self) -> bool:
+        return len(self.sentences) > 0
+
+    def get_text(self) -> str:
+        return self.get_text_no_lf()
+
+    def get_text_no_lf(self) -> str:
+        ret = ""
+        for sen in self.sentences:
+            ret += sen.text + " "
+        return ret
+
+    def get_text_lf(self) -> str:
+        ret = ""
+        for sen in self.sentences:
+            ret += sen.text + get_crlf()
+        return ret
+
+
+class SDUText(BaseModel):
+    raw: str = ""
+    clean: str = ""
+    html_content: str = ""
+    structured_content: Dict = {}
+    lang: SDULanguage = SDULanguage()
+    paragraphs: List[SDUParagraph] = []
+
+    class Config:
+        orm_mode = False
+
+
+class SDUVersion(BaseModel):
+    version: str = ""
+    creation_date: str = ""
+
+    class Config:
+        orm_mode = False
+
+
+class SDULearnset(BaseModel):
+    # dict entry is the class, list are the train entrys for this class
+    version: str = ""
+    text: Dict = {}
+    nlu: Dict = {}
+    nlp: Dict = {}
+    emb: Dict = {}
+    vec_words: Dict = {}
+    vec_sent: Dict = {}
+
+    def set_version(self, version: str):
+        self.version = version
+
+    def reset(self):
+        self.text.clear()
+        self.nlu.clear()
+        self.nlp.clear()
+        self.emb.clear()
+        self.vec_words.clear()
+        self.vec_sent.clear()
+
+    class Config:
+        orm_mode = False
+
+
+class SPKNotaryInput(DocumentInput):
+    """
+    Data input model for Notary.
+
+    Attributes:
+        city: default city to check.
+    """
+
+    city: str = "Bremen"
+
+
+class SPKNotary(BaseModel):
+    """Detected Notary Pydantic Model."""
+
+    sid: Optional[str]
+    last_name: Optional[str]
+    first_name: Optional[str]
+    zip_code: Optional[str]
+    city: Optional[str]
+    office_city: Optional[str]
+    official_location: Optional[str]
+    address: Optional[str]
+    additional_address: Optional[str]
+    title: Optional[str]
+    phone: Optional[str]
+    complete_name_with_official_location: Optional[str]
+    local_city: str = "Bremen"
+    is_local_city: bool
+
+
+class SPKNotaryWinnerDTO(SPKNotary):
+    """DTO, representing the result of service Notary."""
+
+
+class SDUPage(BaseModel):
+    id: int = -1
+    npar: int = 0
+    input: str = ""
+    has_en: bool = False
+    text: SDUText = SDUText()
+    notary: SPKNotary = None
+
+    class Config:
+        orm_mode = False
+
+    def has_text(self):
+        return len(self.text.paragraphs) > 0
+
+    def get_text_default(self):
+        return self.get_text_no_lf()
+
+    def get_text_no_lf(self):
+        ret = ""
+        for par in self.text.paragraphs:
+            for sen in par.sentences:
+                ret += sen.text + " "
+        return ret
+
+    def get_text_no_lf_en(self):
+        ret = ""
+        for par in self.text.paragraphs:
+            for sen in par.sentences_en:
+                ret += sen.text + " "
+        return ret
+
+    def get_text_no_lf_paragraph(self):
+        ret = ""
+        for par in self.text.paragraphs:
+            for sen in par.sentences:
+                ret += sen.text + get_cr_paragraph()
+            ret += get_cr_paragraph()
+        return ret
+
+    def get_text_lf(self, space_before_lf: bool = False):
+        ret = ""
+        for par in self.text.paragraphs:
+            for sen in par.sentences:
+                ret += sen.text
+                if space_before_lf:
+                    ret += get_sentence_seperator()
+                ret += get_crlf()
+        return ret
+
+    def get_text_lf_paragraph(self, space_before_lf: bool = False):
+        ret = ""
+        for par in self.text.paragraphs:
+            for sen in par.sentences:
+                ret += sen.text
+                if space_before_lf:
+                    ret += get_sentence_seperator()
+                ret += get_sentence_seperator()
+            ret += get_cr_paragraph()
+        return ret
+
+    def get_all_sentences_text_list_lf(self):
+        ret = []
+        for par in self.text.paragraphs:
+            for i, sen in enumerate(par.sentences):
+                txt = sen.text
+                if i == len(par.sentences) - 1:
+                    txt = txt + get_cr_paragraph()
+                else:
+                    txt = txt + get_crlf()
+                ret.append(txt)
+        return ret
+
+    def get_text_for_nlp(self):
+        ret = []
+
+        for par in self.text.paragraphs:
+            txt = ""
+            for sen in par.sentences:
+                txt += sen.text.replace("\n", "") + "\n\n"
+            if len(txt) > 1:
+                ret.append(txt)
+        return ret
+
+    def get_text_for_display(self):
+        ret = ""
+        for par in self.text.paragraphs:
+            txt: str
+            if par.semantic_type.__contains__("list"):
+                txt = par.get_text()
+                ret += txt + get_crlf()
+            else:
+                txt = par.get_text()
+                if par.semantic_type.__contains__(
+                    "head"
+                ) or par.semantic_type.__contains__("title"):
+                    ret += get_crlf() + txt + get_crlf()
+                elif len(par.sentences) > 1:
+                    ret += txt + get_cr_paragraph()
+                else:
+                    ret += txt + get_crlf()
+        ret = ret.replace(get_cr_paragraph() + get_crlf(), get_cr_paragraph())
+        return ret
+
+    def get_all_sentences_text_list(self):
+        ret = []
+
+        for par in self.text.paragraphs:
+            if par.semantic_type.__contains__("list"):
+                txt = par.get_text_lf()
+            else:
+                txt = par.get_text()
+            ret.append(txt)
+        return ret
+
+    def get_all_sentences_text_list_no_table_and_lists(self):
+        ret = []
+
+        for par in self.text.paragraphs:
+            txt = ""
+            if par.semantic_type.__contains__("table"):
+                pass
+            elif par.semantic_type.__contains__("list"):
+                pass
+            elif par.semantic_type.__contains__("image_text"):
+                pass
+            elif par.semantic_type.__contains__("imagetext"):
+                pass
+            elif par.semantic_type.__contains__("image"):
+                pass
+            elif par.semantic_type.__contains__("figure"):
+                pass
+            elif par.section.__contains__("footer"):
+                pass
+            elif par.semantic_type.__contains__("title"):
+                pass
+            elif par.semantic_type.__contains__("headline"):
+                pass
+            else:
+                for i, sen in enumerate(par.sentences):
+                    if len(txt) > 0:
+                        txt += " "
+                    txt = txt + sen.text  # getCRParagraph()
+                ret.append(txt)
+        return ret
+
+    def get_all_sentences_text_list_en(self):
+        ret = []
+        for par in self.text.paragraphs:
+            for i, sen in enumerate(par.sentences_en):
+                txt = sen.text
+                if i == len(par.sentences_en) - 1:
+                    txt = txt + get_cr_paragraph()
+                else:
+                    txt = txt + get_sentence_seperator()
+                ret.append(txt)
+        return ret
+
+    def set_input(self, input_text: str):
+        self.input = input_text
+
+
+class SDUData(BaseModel):
+    npages: int = 0  #
+    stats: SDUStatistic = SDUStatistic()  #
+    pages: List[SDUPage] = []
+    converter: List[str] = []
+    email: SDUEmail = SDUEmail()  # email header
+    text: SDUText = SDUText()  # parsed/body only
+    images: List[SDUPageImage] = []
+
+    class Config:
+        orm_mode = False
+
+    def add_page_pre_processing(self, page_pre: SDUPage):
+        if page_pre:
+            self.pages.append(page_pre)
+            self.npages = len(self.pages)
+            page_pre.page = self.npages
+
+    def escaped(self):
+        return html.escape(self.text.html_content)
+
+    async def sanitized(self):
+        return await sanitize(self.text.html_content)
+
+
+class SDUBBox(BaseModel):
+    x0: float = -1
+    y0: float = -1
+    x1: float = -1
+    y1: float = -1
+
+    class Config:
+        orm_mode = False
+
+
+class SDUElement(BaseModel):
+    id: int
+    start: int = -1
+    end: int = -1
+
+    class Config:
+        orm_mode = False
+
+
+class SDUAttachment(BaseModel):
+    id: str = ""
+    name: str = ""
+    path: str = ""
+    meta: Dict = {}
+    text: SDUText = SDUText()
+    charset: str = ""
+    encoding: str = ""
+    disposition: str = ""
+    content_type: str = ""
+    binary: bool = False
+    payload: str = ""
+    status: str = ""
+
+    class Config:
+        orm_mode = False
+
+
+class SDUDimensions(BaseModel):
+    id: int = -1
+    height: float = 0.0
+    width: float = 0.0
+    factor_x: float = 0.0
+    factor_y: float = 0.0
+    rotation: int = 0
+
+    class Config:
+        orm_mode = False
+
+
+class SDUFonts(BaseModel):
+    id: int = -1
+    fontsizes: Dict = {}
+    fonts: List = []
+    avg_fontsize: int = 14
+    small_fontsize: int = 10000
+
+    class Config:
+        orm_mode = False
+
+
+class SDULayout(BaseModel):
+    id: int = -1
+    dimensions: SDUDimensions = SDUDimensions()
+    fonts: SDUFonts = SDUFonts()
+    texttrace: List = []
+    images: List = []
+    drawings: List = []
+    blocks: List[tuple] = []
+    bjson: Dict = {}
+    columns: List[SDUBBox] = []
+    rows: List[SDUBBox] = []
+    layouts: List = []
+    header: SDUBBox = SDUBBox()
+    body: SDUBBox = SDUBBox()
+    footer: SDUBBox = SDUBBox()
+    margin_left: SDUBBox = SDUBBox()
+    margin_right: SDUBBox = SDUBBox()
+
+    class Config:
+        orm_mode = False
+
+
+class SDUContent(BaseModel):
+    attachments: List[SDUAttachment] = []
+    layouts: List[SDULayout] = []
+
+    class Config:
+        orm_mode = False
+
+
 class SentencesInput(BaseModel):
     """
     Input model with sentences
@@ -107,7 +651,7 @@ class SPKLanguageInput(DocumentInput):
     is_short_text: bool = False
 
 
-class SPKLanguageDTO(sdu.SDULanguageDetails):
+class SPKLanguageDTO(SDULanguageDetails):
     """DTO, representing the result of service language."""
 
 
@@ -119,7 +663,7 @@ class TextWithParagraphsGet(BaseModel):
         paragraphs: list of SDUParagraph.
     """
 
-    paragraphs: List[sdu.SDUParagraph]
+    paragraphs: List[SDUParagraph]
 
 
 class TextWithSentencesGet(BaseModel):
@@ -130,7 +674,7 @@ class TextWithSentencesGet(BaseModel):
         sentences: list of SDUSentence.
     """
 
-    sentences: List[sdu.SDUSentence]
+    sentences: List[SDUSentence]
 
 
 class TextWithPagesGet(BaseModel):
@@ -141,7 +685,7 @@ class TextWithPagesGet(BaseModel):
         pages: list of SDUPage.
     """
 
-    pages: List[sdu.SDUPage]
+    pages: List[SDUPage]
 
 
 class SPKSegmentationInput(BaseModel):
@@ -170,9 +714,9 @@ class SPKSegmentationDTO(BaseModel):
         sentences: list of SDUSentence.
     """
 
-    pages: List[sdu.SDUPage] = []
-    paragraphs: List[sdu.SDUParagraph] = []
-    sentences: List[sdu.SDUSentence] = []
+    pages: List[SDUPage] = []
+    paragraphs: List[SDUParagraph] = []
+    sentences: List[SDUSentence] = []
 
 
 class SPKTextCleanInput(DocumentInput):
@@ -320,7 +864,7 @@ class SPKStatisticsInput(DocumentLangInput):
     """Data input model for Statistics."""
 
 
-class SPKStatisticsDTO(sdu.SDUStatistic):
+class SPKStatisticsDTO(SDUStatistic):
     """DTO, representing the result of service Statistics."""
 
 
@@ -328,40 +872,6 @@ class SPKSummaryDTO(wdc.WDCItem):
     """DTO, representing the result of service Summary."""
 
 
-class SPKNotaryInput(DocumentInput):
-    """
-    Data input model for Notary.
-
-    Attributes:
-        city: default city to check.
-    """
-
-    city: str = "Bremen"
-
-
-class SPKNotary(BaseModel):
-    """Detected Notary Pydantic Model."""
-
-    sid: Optional[str]
-    last_name: Optional[str]
-    first_name: Optional[str]
-    zip_code: Optional[str]
-    city: Optional[str]
-    office_city: Optional[str]
-    official_location: Optional[str]
-    address: Optional[str]
-    additional_address: Optional[str]
-    title: Optional[str]
-    phone: Optional[str]
-    complete_name_with_official_location: Optional[str]
-    local_city: str = "Bremen"
-    is_local_city: bool
-
-
-class SPKNotaryWinnerDTO(SPKNotary):
-    """DTO, representing the result of service Notary."""
-
-
 class SPKCountry(BaseModel):
     """Detected Country Pydantic Model."""
 
@@ -425,6 +935,7 @@ class SPKTaxonomyCountriesDTO(BaseModel):
         countries: List of SPKCountries.
         countries_winner: winner object SPKCountry.
     """
+
     countries: List[SPKCountry]
     countries_winner: Optional[SPKCountry]
 
@@ -462,11 +973,13 @@ class AutoMLStatus(BaseModel):
         info: Service status.
         id: UUID model identifier.
         path: The path where model is located
+        model_data: train columns, features, etc
     """
 
     info: str
     id: Optional[uuid.UUID]
     path: Optional[str]
+    model_data: Optional[Dict]
 
 
 class SPKProfileInput(BaseModel):
@@ -537,6 +1050,7 @@ class SPKBuildModelInput(BaseModel):
         create_metadata: Flag indicating whether to create model metadata
         webhook_url: url to custom HTTP back requests.
     """
+
     model_name: str = "kim_pipeline"
     data: List[Dict[str, Any]]
     target_columns: List[str] = ["IMPULSART", "IMPULSKATEGORIE"]
@@ -608,10 +1122,13 @@ class SPKDBBaseDocumentInput(BaseModel):
         uid: document uid
         name: document name.
         mimetype: mimetype.
-        path: path to file.
+        full_file_path: path to file.
         layout_file_path: path to layout file.
         debug_file_path: path to debug file.
         readorder_file_path: path to rearorder file.
+        clean_text_path: path to txt file with clean text.
+        raw_text_path: path to txt file with raw text.
+        html_path: path to txt file with html.
         folder: folder name.
         group_uuid: group identifier.
         size_bytes: size in bytes.
@@ -627,11 +1144,13 @@ class SPKDBBaseDocumentInput(BaseModel):
         data: data.
         project_code: project code.
         npages: count of pages.
+        images: images.
+        pages_layout: layouts.
+        pages_text: pages.
         content: content.
         metadata: metadata.
         description: discription.
         status: document status
-        text: text.
         file: file.
         sdu: Dict of sdu objects.
     """
@@ -639,10 +1158,13 @@ class SPKDBBaseDocumentInput(BaseModel):
     uid: str
     name: str
     mimetype: str = "text/plain"
-    path: str = ""
+    full_file_path: str = ""
     layout_file_path: str = ""
     debug_file_path: str = ""
     readorder_file_path: str = ""
+    clean_text_path: str = ""
+    raw_text_path: str = ""
+    html_path: str = ""
     folder: str = ""
     group_uuid: str = ""
     size_bytes: int = 0
@@ -658,10 +1180,12 @@ class SPKDBBaseDocumentInput(BaseModel):
     data: Optional[SDUData] = None
     project_code: str = ""
     npages: int = 0
+    images: List[SDUPageImage] = []
+    pages_layout: List[SDULayout] = []
+    pages_text: List[SDUPage] = []
     content: Optional[SDUContent] = None
     metadata: Dict = {}
     description: str = ""
-    text: str = ""
     file: Dict = {}
     sdu: Dict = {}
     status: ProcessStatus = ProcessStatus()
